import pickle
import os
import pandas as pd
import numpy as np  

def normalization(data):
    return (data - np.min(data)) / (np.max(data) - np.min(data))

def getRegressionData(x,y,weight):  
    y = np.average(y,axis=1,weights=weight)
    y = y[:,np.newaxis]
    return x,y
    
def getClassificationData(x, y, weight, disturb=0.012):
    assert weight.shape[0] == y.shape[1]
    y = np.sum(y * weight,axis=1)
    
    x_classification = []
    y_classification = []
    
    for i in range(x.shape[0]):
        for j in range(x.shape[0]):
            if y[i] > y[j] * (1 + disturb):
                _y = 1
            elif y[i] < y[j] * (1 - disturb):
                _y = 0
            else:
                continue
            x_classification.append([x[i],x[j]])
            y_classification.append(_y)
            
    x_classification = np.array(x_classification)
    y_classification = np.array(y_classification)
    return x_classification, y_classification


def vectorization_value(knobs, points):
    feature_matrix = np.zeros(shape = (points.shape[0], len(knobs), 2))
    
    for itera in range(points.shape[0]):
        for param_index, param in enumerate(knobs):
            _value = points[itera,param_index]
            if param.__contains__('options'):
                feature_matrix[itera][param_index][0] = 0
                value_index = int(_value * len(param['options']) - 1)
                feature_matrix[itera][param_index][1] = value_index
            else:
                feature_matrix[itera][param_index][0] = 1
                feature_matrix[itera][param_index][1] = (_value - param['range'][0]) / (param['range'][1] - param['range'][0])
    return feature_matrix                
                
def vectorization(knobs,points):
    feature_dim = 0
    for param in knobs:
        if param.__contains__('options'):
            feature_dim = max(feature_dim,param['options'].__len__())
            
    feature_dim += 2
    feature_matrix = np.zeros(shape = (points.shape[0], len(knobs), feature_dim))
    for itera in range(points.shape[0]):
        for param_index, param in enumerate(knobs):
            _value = points[itera,param_index]
            if param.__contains__('options'):
                value_index = int(_value)
                feature_matrix[itera][param_index][value_index] = 1
            else:
                feature_matrix[itera][param_index][-2] = _value
                feature_matrix[itera][param_index][-1] = 1

 
                
    return feature_matrix
                    
def dataFromNumpy(data_path,vec_type='vec'):
    knobs = pickle.load(open(os.path.join(data_path,'knobs.pkl'),'rb'))
    points = pickle.load(open(os.path.join(data_path,'points.pkl'),'rb'))
    scores = pickle.load(open(os.path.join(data_path,'score.pkl'),'rb'))
    bench = pickle.load(open(os.path.join(data_path,'bench.pkl'),'rb'))
    print(type(points))
    print(points.shape)
    if vec_type == "vec":
        feature = vectorization(knobs, points)
    else:
        feature = vectorization_value(knobs, points)
    scores = scores/np.mean(scores,axis=0)*10000
    label = list(bench.keys())
    return feature, scores, list(bench.keys())
