import json
import logging
import functools
import traceback

logger = logging.getLogger(__name__)

CALL_LEVEL = -1
PLACEHOLDER = " " * 4

def funcLog(func):
    """ More general auto logging decorator to function call
    
    Function can return any form of data.
    Handling all exception occured in this function, logging traceback and return None.

    """
    @functools.wraps(func)
    def wrapper(*args, **kw):
        global CALL_LEVEL, PLACEHOLDER
        CALL_LEVEL += 1
        
        if CALL_LEVEL == 0:
            logger.info("{placeholder}[{module}.{func}] << {args} {kw}".format(
                    placeholder = PLACEHOLDER*CALL_LEVEL, 
                    module = func.__module__,
                    func = func.__qualname__,
                    args = ",".join(["{}".format(_arg) for _arg in args]), 
                    kw = ",".join(["{} = {}".format(k,v) for k,v in kw.items()]))) 
        else:
            logger.debug("{placeholder}[{module}.{func}] << {args} {kw}".format(
                    placeholder = PLACEHOLDER*CALL_LEVEL, 
                    module = func.__module__,
                    func = func.__qualname__,
                    args = ",".join(["{}".format(_arg) for _arg in args]), 
                    kw = ",".join(["{} = {}".format(k,v) for k,v in kw.items()]))) 
    
        try:
            out = func(*args,**kw)
            
        except Exception as e:
            logger.critical('[{module}.{func}] {trace}'.format(
                    module = func.__module__,
                    func = func.__qualname__,
                    trace = traceback.format_exc()))
            CALL_LEVEL -= 1
            return None
            
        else:
            if CALL_LEVEL == 0:
                logger.info("{placeholder}[{module}.{func}] >> {out}".format(
                    placeholder = PLACEHOLDER*CALL_LEVEL, 
                    module = func.__module__,
                    func = func.__qualname__,
                    out = out))
            else:
                logger.debug("{placeholder}[{module}.{func}] >> {out}".format(
                    placeholder = PLACEHOLDER*CALL_LEVEL, 
                    module = func.__module__,
                    func = func.__qualname__,
                    out = out))
                
            CALL_LEVEL -= 1
            return out
        
    return wrapper