import numpy as np

def random(searchspace, pts_num: int):
    """ Sampling some points

    Args:
        pts_num (int): Number of points to sampling.
        searchspace(keenopt.searchspace): parameter search space.

    Returns:
        np.array
    """
    return searchspace.getRandomPts(pts_num)


def LatinHypercube(searchspace, pts_num: int):
    """ Sampling some points

    Args:
        pts_num (int): Number of points to sampling.
        searchspace(keenopt.searchspace): parameter search space.

    Returns:
        np.array
    """
    D = searchspace.dim
    N = pts_num
    result = np.empty([N, D])
    temp = np.empty([N])
    d = 1.0 / N
    bounds = []
    int_param = []
    for param_obj in searchspace.param_list:
        if hasattr(param_obj,"options"):
            bounds.append([0,1])
        else:
            bounds.append([param_obj.lb,param_obj.ub])

    for i in range(D):
        if hasattr(searchspace.param_list[i],"options"):
            for j in range(N):
                temp[j] = np.random.randint(
                low=0, high=searchspace.param_list[i].options.__len__(), size = 1)[0]
            np.random.shuffle(temp)
        else:
            if searchspace.param_list[i].dtype == "int" or searchspace.param_list[i].dtype == "long":
                int_param.append(i)
            for j in range(N):
                temp[j] = np.random.uniform(
                    low=j * d, high=(j + 1) * d, size = 1)[0]
            np.random.shuffle(temp)
        for j in range(N):
            result[j, i] = temp[j]
    b = np.array(bounds)
    lower_bounds = b[:,0]
    upper_bounds = b[:,1]
    if np.any(lower_bounds > upper_bounds):
        return None
    np.add(np.multiply(result,
                       (upper_bounds - lower_bounds),
                       out=result),
           lower_bounds,
           out=result)
    result[:,int_param] = np.rint(result[:,int_param])
    return result


def to_unit_cube(searchspace, x):
    """Project to [0, 1]^d from hypercube with bounds lb and ub"""
    if x.shape.__len__() == 1:
        x = np.expand_dims(x, 0)

    xx = np.zeros_like(x)
    D = searchspace.dim
    for i in range(D):
        if hasattr(searchspace.param_list[i],"options"):
            lb = 0
            ub = searchspace.param_list[i].options.__len__() - 1
        else:
            lb = searchspace.param_list[i].lb
            ub = searchspace.param_list[i].ub

        xx[:,i] = (x[:,i] - lb) / (ub - lb)
    return xx

def from_unit_cube(searchspace, x):
    """Project from [0, 1]^d to hypercube with bounds lb and ub"""
    if x.shape.__len__() == 1:
        x = np.expand_dims(x, 0)

    xx = np.zeros_like(x)
    D = searchspace.dim
    for i in range(D):
        if hasattr(searchspace.param_list[i], "options"):
            lb = 0
            ub = searchspace.param_list[i].options.__len__() - 1
            xx[:, i] = x[:, i] * (ub - lb) + lb
            np.rint(xx[:, i])
        else:
            lb = searchspace.param_list[i].lb
            ub = searchspace.param_list[i].ub
            xx[:, i] = x[:, i] * (ub - lb) + lb
    return np.squeeze(xx)