import os
import pickle
import json
import time
import sys
import getopt
import numpy as np
import pandas as pd

from sklearn.ensemble import RandomForestClassifier,RandomForestRegressor
from sklearn.linear_model import LinearRegression
from sklearn.linear_model import ElasticNet
from sklearn.linear_model import Lasso
from sklearn.preprocessing import StandardScaler

from keenopt.searchspace.searchspace import SearchSpace


def ElasticNetRegressor(*data):
    x_train, y_train = data
    regr = ElasticNet(random_state=1)
    regr.fit(x_train, y_train)
    return regr.coef_

def LassoRegressor(*data):
    x_train, y_train = data
    regr = Lasso()
    regr.fit(x_train, y_train)
    return regr.coef_

def LinearRegressor(*data):
    x_train,  y_train = data
    x_train = x_train
    y_train = y_train
    regr = LinearRegression()
    regr.fit(x_train, y_train)
    # print(MAPE(regr.predict(x_train), y_train))
    return regr.coef_


def researchspace(H_points, H_score, knobs, fx_weight):    
    #get target bench score
    score = np.sum(H_score * fx_weight,axis=1)
    
    points = H_points

    knobs = knobs

    columns = []
    for i in range(len(knobs)):
        columns.append(knobs[i]["name"])
    df = pd.DataFrame(columns=columns,data=points)
    df["target"] = score[:len(points)]
    feat_labels = df.columns[:-1]
    x, y = df.iloc[:, :-1].values, df.iloc[:, -1].values
    el_weight = np.absolute(ElasticNetRegressor(x,y))
 
    el_weight = el_weight/np.sum(el_weight)

    total_weight = el_weight

    indices = np.argsort(total_weight)
    indices = indices[-int(np.floor(len(indices)/4)):]

    sensi_knobs = []
    for i in indices:
        sensi_knobs.append(knobs[i])

    parameters = {}
    for knob in sensi_knobs:
        parameters[knob['name']] = knob
        
    searchspace = SearchSpace(parameters)

    return searchspace,indices
