import numpy as np

from collections import Iterable

from keenopt.searchspace.parameter import Parameter
from keenopt.common.log import funcLog

class Options(Parameter):
    @funcLog
    def __init__(self, param_name, dtype, options):
        super().__init__(param_name, dtype)
        self.options = options

        assert isinstance(self.options, Iterable)
    
    
    def randomValue(self, nums):
        return np.random.randint(0,self.options.__len__(),nums)
    
    
    def getConfig(self, value):
        assert value < len(self.options)
        return self._parseDtype(self.options[int(value)])


    def perturb(self, value, perturbation):
        return self.randomValue(1)


    @funcLog
    def perturbGrad(self, value, grad, perturbation):
        """ perturbing value by grad

        Args:
            value (float):  Parameter value.
            grad (float):   Gradient if parameter in this value.

        Returns:
            float
        """
        if grad <= 0:
            return self.randomValue(1)
        else:
            return value


    def distance(self,value1,value2):
        """ Calculate distance between value1 and value 2.

        Returns:
            float
        """
        return 1.0 if value1 != value2 else 0