import math  
import numpy as np

from copy import deepcopy
from collections import Iterable

from keenopt.searchspace.parameter import Parameter
from keenopt.common.log import funcLog


class Sequence(Parameter):
    @funcLog
    def __init__(self, param_name, dtype, options):
        super().__init__(param_name, dtype)
        self.options = options
        self.ub = len(options) - 1
        self.lb = 0
        self.step = 1
        self.range_size = len(options)
        
        assert isinstance(self.options, Iterable)
    
    def randomValue(self, nums):
        return np.random.randint(0,self.options.__len__(),nums)
    
    def getConfig(self, value):
        assert value < len(self.options)
        return self._parseDtype(self.options[int(value)])
    
    def perturb(self, value, perturbation):
        return self.randomValue(1)
    
    @funcLog
    def perturbGrad(self, value, grad, perturbation):
        delta = max(1, int(perturbation * self.range_size / 100 ))
        
        if grad > 0:
            value -= self.step * delta
            value = max(value, self.lb)
        else:
            value += self.step * delta
            value = min(value, self.ub)
        return value
    
    
    def distance(self, value1, value2):
        return np.abs(value1 - value2) / (self.ub - self.lb)