import torch
import torch.nn as nn
import torch.nn.functional as F


def _init_weights(model):
    for name, param in model.named_parameters():
        nn.init.uniform_(param.data, -0.1, 0.1)


class MLPregression(nn.Module):
    def __init__(self, x_dim, fx_dim):
        super(MLPregression, self).__init__()
        self.x_dim = x_dim
        self.fx_dim = fx_dim

        self.common_layer = nn.Sequential(
            nn.Linear(in_features = x_dim, out_features = x_dim * 4, bias = True),
            nn.ReLU(),
            nn.Linear(in_features = x_dim * 4,out_features = x_dim * 4, bias = True),
            nn.ReLU(),
            nn.Linear(in_features = x_dim * 4,out_features = x_dim * 2, bias = True),
            nn.ReLU(),
        )
        
        self.unique_layer = []
        for _ in range(self.fx_dim):
            self.unique_layer.append(nn.Sequential(
                nn.Linear(in_features = x_dim * 2,out_features = x_dim, bias = True),
                #nn.ReLU(),
                nn.Linear(in_features = x_dim,out_features = 1)
            ))

        self.common_layer.apply(_init_weights)
        for l in self.unique_layer:
            l.apply(_init_weights)

    def forward(self, x):
        common_feature = self.common_layer(x)

        output = torch.empty(common_feature.shape[0],0)
        
        for i in range(self.fx_dim):
            _out = self.unique_layer[i](common_feature)
            output = torch.cat((output, _out), 1)
        return output
